@@ -233,7 +233,7 @@
           theseArgs = theseArgs.concat([task.data]);
           task = task.code;
         }
-        var taskFunc = eval("(" + task + ")");
+        var taskFunc = tryEval(task);
         if (typeof(taskFunc) !== "function") {
           throw new Error("Task must be a function! Source:\n" + task);
         }
@@ -242,6 +242,31 @@
     }
   }
 
+  // Attempt eval() both with and without enclosing in parentheses.
+  // Note that enclosing coerces a function declaration into
+  // an expression that eval() can parse
+  // (otherwise, a SyntaxError is thrown)
+  function tryEval(code) {
+    var result = null;
+    try {
+      result = eval(code);
+    } catch(error) {
+      if (!error instanceof SyntaxError) {
+        throw error;
+      }
+      try {
+        result = eval("(" + code + ")");
+      } catch(e) {
+        if (e instanceof SyntaxError) {
+          throw error;
+        } else {
+          throw e;
+        }
+      }
+    }
+    return result;
+  }
+
   function initSizing(el) {
     var sizing = sizingPolicy(el);
     if (!sizing)
@@ -489,6 +514,7 @@
       // supported natively by Shiny at the time of this writing.
 
       shinyBinding.renderValue = function(el, data) {
+        Shiny.renderDependencies(data.deps);
         // Resolve strings marked as javascript literals to objects
         if (!(data.evals instanceof Array)) data.evals = [data.evals];
         for (var i = 0; data.evals && i < data.evals.length; i++) {
@@ -511,10 +537,9 @@
               el.offsetHeight);
             elementData(el, "init_result", result);
           }
-          evalAndRun(data.jsHooks.render, elementData(el, "init_result"), [el, data.x]);
         }
-        Shiny.renderDependencies(data.deps);
         bindingDef.renderValue(el, data.x, elementData(el, "init_result"));
+        evalAndRun(data.jsHooks.render, elementData(el, "init_result"), [el, data.x]);
       };
 
       // Only override resize if bindingDef implements it
@@ -565,7 +590,10 @@
         }
 
         if (binding.resize) {
-          var lastSize = {};
+          var lastSize = {
+            w: sizeObj ? sizeObj.getWidth() : el.offsetWidth,
+            h: sizeObj ? sizeObj.getHeight() : el.offsetHeight
+          };
           var resizeHandler = function(e) {
             var size = {
               w: sizeObj ? sizeObj.getWidth() : el.offsetWidth,
@@ -632,7 +660,34 @@
   }
 
   // Wait until after the document has loaded to render the widgets.
-  if (document.addEventListener) {
+  if (shinyMode && window.jQuery) {
+    /*
+    / Shiny 1.4.0 bumps jQuery from 1.x to 3.x, which means jQuery's
+    / on-ready handler (i.e., $(fn)) is now asyncronous (i.e., it now
+    / really means $(setTimeout(fn)). https://jquery.com/upgrade-guide/3.0/#breaking-change-document-ready-handlers-are-now-asynchronous
+    /
+    / In order to ensure the order of execution of on-ready handlers
+    / remains consistent with how it's been in the past, a static render
+    / in Shiny is now scheduled via $().
+    /
+    / This is not a long term solution: it just preserves the current order
+    / of execution. Part of that ordering is to ensure initShiny executes
+    / _after_ staticRender, which is both right and wrong:
+    / * It's wrong because, when initShiny executes, it registers methods
+    / like Shiny.onInputChange that widget authors would expect to be available
+    / during a staticRender.
+    / * It's also 'right' because initShiny currently (as of v1.4.0) wants
+    / to execute after user code so that it can bind to any dynamically
+    / created elements.
+    /
+    / A longer term solution might be to make changes to Shiny so that
+    / these methods are available before the binding takes place, and then
+    / the ordering would be: register methods -> static render -> bind.
+    */
+    window.jQuery(function() {
+      window.HTMLWidgets.staticRender();
+    });
+  } else if (document.addEventListener) {
     document.addEventListener("DOMContentLoaded", function() {
       document.removeEventListener("DOMContentLoaded", arguments.callee, false);
       window.HTMLWidgets.staticRender();
@@ -729,7 +784,7 @@
       if (o !== null && typeof o === "object" && part in o) {
         if (i == (l - 1)) { // if we are at the end of the line then evalulate
           if (typeof o[part] === "string")
-            o[part] = eval("(" + o[part] + ")");
+            o[part] = tryEval(o[part]);
         } else { // otherwise continue to next embedded object
           o = o[part];
         }
